import timeit
import random
import max_product_naive
import max_product_fast1


def rand_list(N, M):
    lst = []
    for j in range(N):
        x = random.randint(0, M)
        lst.append(x)
    return lst


def function_wrapper(func, *args, **kwargs):
    def wrapped():
        return func(*args, **kwargs)
    return wrapped


def performance_test(num_tests, min_len, max_len, step):
    print ("Input  \t Algo 1 \t Algo 2 \t Ratio")

    # generate random inputs of length N in a given range
    for N in range (min_len, max_len+step, step):
        lst = rand_list(N, 100)

        # Get min time from num_test repetitions of algorithm 1
        func_naive = function_wrapper(max_product_naive.max_product, lst)
        naive_time = timeit.timeit(func_naive, number=num_tests)

        # Get min time from num_test repetitions of algorithm 2
        func_fast = function_wrapper(max_product_fast1.max_product, lst)
        fast_time = timeit.timeit(func_fast, number=num_tests)

        # Print input size, times, improvement
        print("{0:d} \t {1:7.6f} \t {2:7.6f} \t{3:10.3f}".format(N, naive_time, fast_time, naive_time/fast_time))


if __name__ == "__main__":
    num_tests = 10
    min_N = 500
    max_N = 5000
    step = 500
    performance_test(num_tests, min_N, max_N, step)